@@ -9,9 +9,9 @@ typedef ::mdds::segment_tree<long, string> db_type;
 
 struct string_printer : public unary_function<string*, void>
 {
-    void operator() (const string* p) const
+    void operator() (const string& s) const
     {
-        cout << "search hit: " << *p << endl;
+        cout << "search hit: " << s << endl;
     }
 };
 
@@ -23,9 +23,9 @@ int main()
     string C("C");
 
     // Insert data into the tree.
-    db.insert(0,  10, &A);
-    db.insert(2,  20, &B);
-    db.insert(10, 15, &C);
+    db.insert(0,  10, A);
+    db.insert(2,  20, B);
+    db.insert(10, 15, C);
 
     // Don't forget to build it before calling search().
     db.build_tree();
