@@ -53,6 +53,14 @@
  */
 #define MDDS_N_ELEMENTS(name) sizeof(name)/sizeof(name[0])
 
+#ifdef __GNUC__
+    #define MDDS_DEPRECATED __attribute__ ((deprecated))
+#elif defined(_MSC_VER)
+    #define MDDS_DEPRECATED __declspec(deprecated)
+#else
+    #define MDDS_DEPRECATED
+#endif
+
 namespace mdds {
 
 class general_error : public ::std::exception
@@ -87,6 +95,12 @@ public:
     type_error(const std::string& msg) : general_error(msg) {}
 };
 
+class integrity_error : public general_error
+{
+public:
+    integrity_error(const std::string& msg) : general_error(msg) {}
+};
+
 template<typename T, typename ...Args>
 std::unique_ptr<T> make_unique(Args&& ...args)
 {
