@@ -1,6 +1,6 @@
 /*************************************************************************
  *
- * Copyright (c) 2012-2014 Kohei Yoshida
+ * Copyright (c) 2012-2015 Kohei Yoshida
  *
  * Permission is hereby granted, free of charge, to any person
  * obtaining a copy of this software and associated documentation
@@ -25,7 +25,7 @@
  *
  ************************************************************************/
 
-#include "test_global.hpp"
+#include "test_global.hpp" // This must be the first header to be included.
 
 #define MDDS_MULTI_TYPE_VECTOR_DEBUG 1
 #include <mdds/multi_type_vector.hpp>
@@ -35,12 +35,10 @@
 #include <mdds/multi_type_vector_custom_func3.hpp>
 
 #include <cassert>
+#include <memory>
 #include <sstream>
 #include <vector>
 
-#include <boost/ptr_container/ptr_vector.hpp>
-#include <boost/noncopyable.hpp>
-
 #define ARRAY_SIZE(x) sizeof(x)/sizeof(x[0])
 
 using namespace std;
@@ -76,6 +74,7 @@ struct muser_cell
     muser_cell() : value(0.0) {}
     muser_cell(const muser_cell& r) : value(r.value) {}
     muser_cell(double _v) : value(_v) {}
+    ~muser_cell() {}
 };
 
 struct date
@@ -89,14 +88,18 @@ struct date
 };
 
 template<typename T>
-class cell_pool : boost::noncopyable
+class cell_pool
 {
-    boost::ptr_vector<T> m_pool;
+    std::vector<std::unique_ptr<T>> m_pool;
 public:
+    cell_pool() = default;
+    cell_pool(const cell_pool&) = delete;
+    cell_pool& operator=(const cell_pool&) = delete;
+
     T* construct()
     {
-        m_pool.push_back(new T);
-        return &m_pool.back();
+        m_pool.emplace_back(new T);
+        return m_pool.back().get();
     }
 
     void clear() { m_pool.clear(); }
@@ -105,6 +108,7 @@ public:
 class user_cell_pool : public cell_pool<user_cell>
 {
 public:
+
     user_cell* construct(double val)
     {
         user_cell* p = cell_pool<user_cell>::construct();
@@ -118,8 +122,8 @@ typedef mdds::mtv::managed_element_block<element_type_muser_block, muser_cell> m
 typedef mdds::mtv::default_element_block<element_type_fruit_block, my_fruit_type> fruit_block;
 typedef mdds::mtv::default_element_block<element_type_date_block, date> date_block;
 
-MDDS_MTV_DEFINE_ELEMENT_CALLBACKS_PTR(user_cell, element_type_user_block, NULL, user_cell_block)
-MDDS_MTV_DEFINE_ELEMENT_CALLBACKS_PTR(muser_cell, element_type_muser_block, NULL, muser_cell_block)
+MDDS_MTV_DEFINE_ELEMENT_CALLBACKS_PTR(user_cell, element_type_user_block, nullptr, user_cell_block)
+MDDS_MTV_DEFINE_ELEMENT_CALLBACKS_PTR(muser_cell, element_type_muser_block, nullptr, muser_cell_block)
 MDDS_MTV_DEFINE_ELEMENT_CALLBACKS(my_fruit_type, element_type_fruit_block, unknown_fruit, fruit_block)
 MDDS_MTV_DEFINE_ELEMENT_CALLBACKS(date, element_type_date_block, date(), date_block)
 
@@ -149,21 +153,21 @@ void mtv_test_types()
 
     // Basic types
     ct = mtv_type::get_element_type(double(12.3));
-    assert(ct == mtv::element_type_numeric);
+    assert(ct == mtv::element_type_double);
     ct = mtv_type::get_element_type(string());
     assert(ct == mtv::element_type_string);
-    ct = mtv_type::get_element_type(static_cast<unsigned long>(12));
-    assert(ct == mtv::element_type_ulong);
+    ct = mtv_type::get_element_type(static_cast<uint64_t>(12));
+    assert(ct == mtv::element_type_uint64);
     ct = mtv_type::get_element_type(true);
     assert(ct == mtv::element_type_boolean);
     ct = mtv_type::get_element_type(false);
     assert(ct == mtv::element_type_boolean);
 
     // Custom cell type
-    user_cell* p = NULL;
+    user_cell* p = nullptr;
     ct = mtv_type::get_element_type(p);
     assert(ct == element_type_user_block && ct >= mtv::element_type_user_start);
-    ct = mtv_type::get_element_type(static_cast<muser_cell*>(NULL));
+    ct = mtv_type::get_element_type(static_cast<muser_cell*>(nullptr));
     assert(ct == element_type_muser_block && ct >= mtv::element_type_user_start);
     ct = mtv_fruit_type::get_element_type(unknown_fruit);
     assert(ct == element_type_fruit_block && ct >= mtv::element_type_user_start);
@@ -335,7 +339,7 @@ void mtv_test_basic()
         // Get empty value.
         mtv_type db(1);
         user_cell* p = db.get<user_cell*>(0);
-        assert(p == NULL);
+        assert(p == nullptr);
     }
 }
 
@@ -414,7 +418,7 @@ void mtv_test_managed_block()
         db.set_empty(0, 1);
 
         db.set(0, new muser_cell(6.0));
-        db.set(1, static_cast<unsigned long>(12));
+        db.set(1, static_cast<uint64_t>(12));
         db.set_empty(0, 2);
     }
 
@@ -681,7 +685,7 @@ void mtv_test_managed_block()
         mtv_type db(4);
         db.set(0, 1.1);
         db.set(1, new muser_cell(1.0));
-        db.set(2, static_cast<unsigned long>(2));
+        db.set(2, static_cast<uint64_t>(2));
         db.set(3, new muser_cell(3.0));
         assert(db.block_size() == 4);
         assert(db.size() == 4);
@@ -702,15 +706,15 @@ void mtv_test_managed_block()
         db.set(2, 1.2);
         db.set(3, new muser_cell(3.0));
         db.set(4, new muser_cell(4.0));
-        unsigned long vals[] = { 5, 6, 7 };
-        const unsigned long* p = &vals[0];
+        uint64_t vals[] = { 5, 6, 7 };
+        const uint64_t* p = &vals[0];
         db.set(1, p, p+3);
     }
 
     {
         // set_cells() across multiple blocks, part 2.
         mtv_type db(6);
-        db.set(0, static_cast<unsigned long>(12));
+        db.set(0, static_cast<uint64_t>(12));
         db.set(1, new muser_cell(1.0));
         db.set(2, new muser_cell(2.0));
         db.set(3, 1.2);
@@ -729,14 +733,14 @@ void mtv_test_managed_block()
     {
         // set_cell() to merge 3 blocks.
         mtv_type db(6);
-        db.set(0, static_cast<unsigned long>(12));
+        db.set(0, static_cast<uint64_t>(12));
         db.set(1, new muser_cell(1.0));
         db.set(2, new muser_cell(2.0));
         db.set(3, 1.2);
         db.set(4, new muser_cell(3.0));
         db.set(5, new muser_cell(4.0));
         assert(db.block_size() == 4);
-        assert(db.get<unsigned long>(0) == 12);
+        assert(db.get<uint64_t>(0) == 12);
         assert(db.get<muser_cell*>(1)->value == 1.0);
         assert(db.get<muser_cell*>(2)->value == 2.0);
         assert(db.get<double>(3) == 1.2);
@@ -745,7 +749,7 @@ void mtv_test_managed_block()
 
         db.set(3, new muser_cell(5.0)); // merge blocks.
         assert(db.block_size() == 2);
-        assert(db.get<unsigned long>(0) == 12);
+        assert(db.get<uint64_t>(0) == 12);
         assert(db.get<muser_cell*>(1)->value == 1.0);
         assert(db.get<muser_cell*>(2)->value == 2.0);
         assert(db.get<muser_cell*>(3)->value == 5.0);
@@ -756,7 +760,7 @@ void mtv_test_managed_block()
     {
         // set_cell() to merge 2 blocks.
         mtv_type db(3);
-        db.set(0, static_cast<unsigned long>(23));
+        db.set(0, static_cast<uint64_t>(23));
         db.set(1, new muser_cell(2.1));
         db.set(2, new muser_cell(3.1));
 
@@ -960,6 +964,36 @@ void mtv_test_managed_block()
         delete p1;
         delete p2;
     }
+
+    {
+        mtv_type db(5);
+
+        db.set(1, new muser_cell(1.1));
+        db.set(2, new muser_cell(1.2));
+        db.set(3, new muser_cell(1.3));
+
+        db.set(1, 2.1); // Don't leak the overwritten muser_cell instance.
+        db.set(2, 2.2); // ditto
+    }
+
+    {
+        mtv_type db(4);
+        db.set(0, new muser_cell(1.1));
+        db.set(1, new muser_cell(1.2));
+        db.set(2, new muser_cell(1.3));
+
+        db.set(2, 2.1); // Don't leak the overwritten muser_cell instance.
+        db.set(1, 2.0); // ditto
+    }
+
+    {
+        mtv_type db(8);
+        db.set(3, new muser_cell(1.1));
+        db.set(4, new muser_cell(1.2));
+        db.set(5, 1.3);
+
+        db.set(4, 2.2); // Overwrite muser_cell and don't leak.
+    }
 }
 
 void mtv_test_custom_block_func1()
@@ -1260,8 +1294,8 @@ void mtv_test_transfer()
     db2 = mtv_type(10);
     db1.set(0, new muser_cell(2.1));
     db1.set(1, new muser_cell(2.2));
-    db1.set(2, char('a'));
-    db1.set(3, char('b'));
+    db1.set(2, int8_t('a'));
+    db1.set(3, int8_t('b'));
     db2.set(0, true);
     db2.set(1, false);
 
@@ -1272,7 +1306,7 @@ void mtv_test_transfer()
     ++it;
     assert(it != db1.end());
     assert(it->size == 1);
-    assert(it->type == mtv::element_type_char);
+    assert(it->type == mtv::element_type_int8);
     ++it;
     assert(it != db1.end());
     assert(it->size == 6);
@@ -1280,7 +1314,7 @@ void mtv_test_transfer()
     ++it;
     assert(it == db1.end());
     assert(db1.block_size() == 3);
-    assert(db1.get<char>(3) == 'b');
+    assert(db1.get<int8_t>(3) == 'b');
 
     assert(db2.block_size() == 4);
     assert(db2.get<bool>(0) == true);
@@ -1292,7 +1326,7 @@ void mtv_test_transfer()
     assert(db2.is_empty(6));
     assert(db2.get<muser_cell*>(7)->value == 2.1);
     assert(db2.get<muser_cell*>(8)->value == 2.2);
-    assert(db2.get<char>(9) == 'a');
+    assert(db2.get<int8_t>(9) == 'a');
 
     // Multi-block transfer to the middle part of destination block.
     db1 = mtv_type(10);
@@ -1395,6 +1429,152 @@ void mtv_test_swap()
     db2.set(1, string("B"));
 
     db1.swap(2, 3, db2, 0);
+
+    // swap blocks of equal size, one managed, and one default.
+
+    db1.clear();
+    db1.resize(10);
+    db2.clear();
+    db2.resize(10);
+
+    db1.set(3, 2.1);
+    db1.set(4, 2.2);
+    db1.set(5, 2.3);
+
+    db2.set(3, new muser_cell(3.1));
+    db2.set(4, new muser_cell(3.2));
+    db2.set(5, new muser_cell(3.3));
+
+    db2.swap(3, 5, db1, 3);
+
+    assert(db1.size() == 10);
+    assert(db1.block_size() == 3);
+    assert(db2.size() == 10);
+    assert(db2.block_size() == 3);
+
+    assert(db1.get<muser_cell*>(3)->value == 3.1);
+    assert(db1.get<muser_cell*>(4)->value == 3.2);
+    assert(db1.get<muser_cell*>(5)->value == 3.3);
+    assert(db2.get<double>(3) == 2.1);
+    assert(db2.get<double>(4) == 2.2);
+    assert(db2.get<double>(5) == 2.3);
+
+    db2.swap(3, 5, db1, 3);
+
+    assert(db1.get<double>(3) == 2.1);
+    assert(db1.get<double>(4) == 2.2);
+    assert(db1.get<double>(5) == 2.3);
+    assert(db2.get<muser_cell*>(3)->value == 3.1);
+    assert(db2.get<muser_cell*>(4)->value == 3.2);
+    assert(db2.get<muser_cell*>(5)->value == 3.3);
+
+    // Same as above, except that the source segment splits the block into 2.
+
+    db1.clear();
+    db1.resize(10);
+    db2.clear();
+    db2.resize(10);
+
+    db1.set(3, 2.1);
+    db1.set(4, 2.2);
+
+    db2.set(3, new muser_cell(3.1));
+    db2.set(4, new muser_cell(3.2));
+    db2.set(5, new muser_cell(3.3));
+
+    db2.swap(3, 4, db1, 3);
+
+    // Another scenario that used to crash on double delete.
+
+    db1.clear();
+    db1.resize(10);
+    db2.clear();
+    db2.resize(10);
+
+    db1.set(2, new muser_cell(4.1));
+    db1.set(3, 4.2);
+    db1.set(4, new muser_cell(4.3));
+
+    db2.set(3, new muser_cell(6.1));
+    db2.set(4, 6.2);
+    db2.set(5, 6.3);
+
+    assert(db1.get<muser_cell*>(2)->value == 4.1);
+    assert(db1.get<double>(3) == 4.2);
+    assert(db1.get<muser_cell*>(4)->value == 4.3);
+
+    assert(db2.get<muser_cell*>(3)->value == 6.1);
+    assert(db2.get<double>(4) == 6.2);
+    assert(db2.get<double>(5) == 6.3);
+
+    db2.swap(4, 4, db1, 4);
+
+    assert(db1.get<muser_cell*>(2)->value == 4.1);
+    assert(db1.get<double>(3) == 4.2);
+    assert(db1.get<double>(4) == 6.2);
+
+    assert(db2.get<muser_cell*>(3)->value == 6.1);
+    assert(db2.get<muser_cell*>(4)->value == 4.3);
+    assert(db2.get<double>(5) == 6.3);
+
+    // One more on double deletion...
+
+    db1.clear();
+    db1.resize(10);
+    db2.clear();
+    db2.resize(10);
+
+    db1.set(0, 2.1);
+    db1.set(1, 2.2);
+    db1.set(2, 2.3);
+    db1.set(3, new muser_cell(4.5));
+
+    db2.set(2, new muser_cell(3.1));
+    db2.set(3, new muser_cell(3.2));
+    db2.set(4, new muser_cell(3.3));
+
+    db1.swap(2, 2, db2, 3);
+
+    assert(db1.get<double>(0) == 2.1);
+    assert(db1.get<double>(1) == 2.2);
+    assert(db1.get<muser_cell*>(2)->value == 3.2);
+    assert(db1.get<muser_cell*>(3)->value == 4.5);
+
+    assert(db2.get<muser_cell*>(2)->value == 3.1);
+    assert(db2.get<double>(3) == 2.3);
+    assert(db2.get<muser_cell*>(4)->value == 3.3);
+
+    assert(db1.check_block_integrity());
+    assert(db2.check_block_integrity());
+}
+
+void mtv_test_swap_2()
+{
+    stack_printer __stack_printer__("::mtv_test_swap_2");
+    mtv3_type db1(3), db2(3);
+
+    db1.set(0, new muser_cell(1.1));
+    db1.set(1, new muser_cell(1.2));
+
+    db2.set(0, 1.2);
+    db2.set(1, std::string("foo"));
+
+    // Repeat the same swap twice.
+    db1.swap(0, 1, db2, 0);
+    assert(db1.check_block_integrity());
+    assert(db2.check_block_integrity());
+    assert(db2.get<muser_cell*>(0)->value == 1.1);
+    assert(db2.get<muser_cell*>(1)->value == 1.2);
+    assert(db1.get<double>(0) == 1.2);
+    assert(db1.get<std::string>(1) == "foo");
+
+    db1.swap(0, 1, db2, 0);
+    assert(db1.check_block_integrity());
+    assert(db2.check_block_integrity());
+    assert(db1.get<muser_cell*>(0)->value == 1.1);
+    assert(db1.get<muser_cell*>(1)->value == 1.2);
+    assert(db2.get<double>(0) == 1.2);
+    assert(db2.get<std::string>(1) == "foo");
 }
 
 void mtv_test_custom_block_func3()
@@ -1515,17 +1695,26 @@ void mtv_test_construction_with_array()
 
 int main (int argc, char **argv)
 {
-    mtv_test_types();
-    mtv_test_block_identifier();
-    mtv_test_basic();
-    mtv_test_equality();
-    mtv_test_managed_block();
-    mtv_test_custom_block_func1();
-    mtv_test_transfer();
-    mtv_test_swap();
-    mtv_test_custom_block_func3();
-    mtv_test_release();
-    mtv_test_construction_with_array();
+    try
+    {
+        mtv_test_types();
+        mtv_test_block_identifier();
+        mtv_test_basic();
+        mtv_test_equality();
+        mtv_test_managed_block();
+        mtv_test_custom_block_func1();
+        mtv_test_transfer();
+        mtv_test_swap();
+        mtv_test_swap_2();
+        mtv_test_custom_block_func3();
+        mtv_test_release();
+        mtv_test_construction_with_array();
+    }
+    catch (const std::exception& e)
+    {
+        cout << "Test failed: " << e.what() << endl;
+        return EXIT_FAILURE;
+    }
 
     cout << "Test finished successfully!" << endl;
     return EXIT_SUCCESS;
